var jsutils_isPromise = require("./jsutils/isPromise.js");

var type_validate = require("./type/validate.js");

var validation_validate = require("./validation/validate.js");

var execution_execute = require("./execution/execute.js");

var graphql_web = require("@0no-co/graphql.web");

function graphqlImpl(args) {
  const {schema, source, rootValue, contextValue, variableValues, operationName, fieldResolver, typeResolver} = args;
  const schemaValidationErrors = type_validate.validateSchema(schema);
  if (schemaValidationErrors.length > 0) {
    return {
      errors: schemaValidationErrors
    };
  }
  let document;
  try {
    document = graphql_web.parse(source);
  } catch (syntaxError) {
    return {
      errors: [ syntaxError ]
    };
  }
  const validationErrors = validation_validate.validate(schema, document);
  if (validationErrors.length > 0) {
    return {
      errors: validationErrors
    };
  }
  return execution_execute.execute({
    schema,
    document,
    rootValue,
    contextValue,
    variableValues,
    operationName,
    fieldResolver,
    typeResolver
  });
}

exports.graphql = function graphql(args) {
  return new Promise((resolve => resolve(graphqlImpl(args))));
};

exports.graphqlSync = function graphqlSync(args) {
  const result = graphqlImpl(args);
  if (jsutils_isPromise.isPromise(result)) {
    throw new Error("GraphQL execution failed to complete synchronously.");
  }
  return result;
};
