var type_directives = require("../type/directives.js");

var type_schema = require("../type/schema.js");

var validation_validate = require("../validation/validate.js");

var utilities_extendSchema = require("./extendSchema.js");

var graphql_web = require("@0no-co/graphql.web");

function buildASTSchema(documentAST, options) {
  if (!0 !== (null == options ? void 0 : options.assumeValid) && !0 !== (null == options ? void 0 : options.assumeValidSDL)) {
    validation_validate.assertValidSDL(documentAST);
  }
  const emptySchemaConfig = {
    description: void 0,
    types: [],
    directives: [],
    extensions: Object.create(null),
    extensionASTNodes: [],
    assumeValid: !1
  };
  const config = utilities_extendSchema.extendSchemaImpl(emptySchemaConfig, documentAST, options);
  if (null == config.astNode) {
    for (const type of config.types) {
      switch (type.name) {
       case "Query":
        config.query = type;
        break;

       case "Mutation":
        config.mutation = type;
        break;

       case "Subscription":
        config.subscription = type;
      }
    }
  }
  const directives = [ ...config.directives, ...type_directives.specifiedDirectives.filter((stdDirective => config.directives.every((directive => directive.name !== stdDirective.name)))) ];
  return new type_schema.GraphQLSchema({
    ...config,
    directives
  });
}

exports.buildASTSchema = buildASTSchema;

exports.buildSchema = function buildSchema(source, options) {
  return buildASTSchema(graphql_web.parse(source, {
    noLocation: null == options ? void 0 : options.noLocation,
    allowLegacyFragmentVariables: null == options ? void 0 : options.allowLegacyFragmentVariables
  }), {
    assumeValidSDL: null == options ? void 0 : options.assumeValidSDL,
    assumeValid: null == options ? void 0 : options.assumeValid
  });
};
