import { specifiedDirectives } from "../type/directives.mjs";

import { GraphQLSchema } from "../type/schema.mjs";

import { assertValidSDL } from "../validation/validate.mjs";

import { extendSchemaImpl } from "./extendSchema.mjs";

import { parse } from "@0no-co/graphql.web";

function buildASTSchema(documentAST, options) {
  if (!0 !== (null == options ? void 0 : options.assumeValid) && !0 !== (null == options ? void 0 : options.assumeValidSDL)) {
    assertValidSDL(documentAST);
  }
  const emptySchemaConfig = {
    description: void 0,
    types: [],
    directives: [],
    extensions: Object.create(null),
    extensionASTNodes: [],
    assumeValid: !1
  };
  const config = extendSchemaImpl(emptySchemaConfig, documentAST, options);
  if (null == config.astNode) {
    for (const type of config.types) {
      switch (type.name) {
       case "Query":
        config.query = type;
        break;

       case "Mutation":
        config.mutation = type;
        break;

       case "Subscription":
        config.subscription = type;
      }
    }
  }
  const directives = [ ...config.directives, ...specifiedDirectives.filter((stdDirective => config.directives.every((directive => directive.name !== stdDirective.name)))) ];
  return new GraphQLSchema({
    ...config,
    directives
  });
}

function buildSchema(source, options) {
  return buildASTSchema(parse(source, {
    noLocation: null == options ? void 0 : options.noLocation,
    allowLegacyFragmentVariables: null == options ? void 0 : options.allowLegacyFragmentVariables
  }), {
    assumeValidSDL: null == options ? void 0 : options.assumeValidSDL,
    assumeValid: null == options ? void 0 : options.assumeValid
  });
}

export { buildASTSchema, buildSchema };
