var jsutils_invariant = require("../jsutils/invariant.js");

var jsutils_keyMap = require("../jsutils/keyMap.js");

var type_definition = require("../type/definition.js");

var graphql_web = require("@0no-co/graphql.web");

function isMissingVariable(valueNode, variables) {
  return valueNode.kind === graphql_web.Kind.VARIABLE && (null == variables || void 0 === variables[valueNode.name.value]);
}

exports.valueFromAST = function valueFromAST(valueNode, type, variables) {
  if (!valueNode) {
    return;
  }
  if (valueNode.kind === graphql_web.Kind.VARIABLE) {
    const variableName = valueNode.name.value;
    if (null == variables || void 0 === variables[variableName]) {
      return;
    }
    const variableValue = variables[variableName];
    if (null === variableValue && type_definition.isNonNullType(type)) {
      return;
    }
    return variableValue;
  }
  if (type_definition.isNonNullType(type)) {
    if (valueNode.kind === graphql_web.Kind.NULL) {
      return;
    }
    return valueFromAST(valueNode, type.ofType, variables);
  }
  if (valueNode.kind === graphql_web.Kind.NULL) {
    return null;
  }
  if (type_definition.isListType(type)) {
    const itemType = type.ofType;
    if (valueNode.kind === graphql_web.Kind.LIST) {
      const coercedValues = [];
      for (const itemNode of valueNode.values) {
        if (isMissingVariable(itemNode, variables)) {
          if (type_definition.isNonNullType(itemType)) {
            return;
          }
          coercedValues.push(null);
        } else {
          const itemValue = valueFromAST(itemNode, itemType, variables);
          if (void 0 === itemValue) {
            return;
          }
          coercedValues.push(itemValue);
        }
      }
      return coercedValues;
    }
    const coercedValue = valueFromAST(valueNode, itemType, variables);
    if (void 0 === coercedValue) {
      return;
    }
    return [ coercedValue ];
  }
  if (type_definition.isInputObjectType(type)) {
    if (valueNode.kind !== graphql_web.Kind.OBJECT) {
      return;
    }
    const coercedObj = Object.create(null);
    const fieldNodes = jsutils_keyMap.keyMap(valueNode.fields, (field => field.name.value));
    for (const field of Object.values(type.getFields())) {
      const fieldNode = fieldNodes[field.name];
      if (!fieldNode || isMissingVariable(fieldNode.value, variables)) {
        if (void 0 !== field.defaultValue) {
          coercedObj[field.name] = field.defaultValue;
        } else if (type_definition.isNonNullType(field.type)) {
          return;
        }
        continue;
      }
      const fieldValue = valueFromAST(fieldNode.value, field.type, variables);
      if (void 0 === fieldValue) {
        return;
      }
      coercedObj[field.name] = fieldValue;
    }
    return coercedObj;
  }
  if (type_definition.isLeafType(type)) {
    let result;
    try {
      result = type.parseLiteral(valueNode, variables);
    } catch (_error) {
      return;
    }
    if (void 0 === result) {
      return;
    }
    return result;
  }
  jsutils_invariant.invariant(!1);
};
