import { didYouMean } from "../../jsutils/didYouMean.mjs";

import { naturalCompare } from "../../jsutils/naturalCompare.mjs";

import { suggestionList } from "../../jsutils/suggestionList.mjs";

import { isAbstractType, isInterfaceType, isObjectType } from "../../type/definition.mjs";

import { GraphQLError } from "@0no-co/graphql.web";

function FieldsOnCorrectTypeRule(context) {
  return {
    Field(node) {
      const type = context.getParentType();
      if (type) {
        if (!context.getFieldDef()) {
          const schema = context.getSchema();
          const fieldName = node.name.value;
          let suggestion = didYouMean("to use an inline fragment on", function getSuggestedTypeNames(schema, type, fieldName) {
            if (!isAbstractType(type)) {
              return [];
            }
            const suggestedTypes = new Set;
            const usageCount = Object.create(null);
            for (const possibleType of schema.getPossibleTypes(type)) {
              if (!possibleType.getFields()[fieldName]) {
                continue;
              }
              suggestedTypes.add(possibleType);
              usageCount[possibleType.name] = 1;
              for (const possibleInterface of possibleType.getInterfaces()) {
                var _usageCount$possibleI;
                if (!possibleInterface.getFields()[fieldName]) {
                  continue;
                }
                suggestedTypes.add(possibleInterface);
                usageCount[possibleInterface.name] = (null !== (_usageCount$possibleI = usageCount[possibleInterface.name]) && void 0 !== _usageCount$possibleI ? _usageCount$possibleI : 0) + 1;
              }
            }
            return [ ...suggestedTypes ].sort(((typeA, typeB) => {
              const usageCountDiff = usageCount[typeB.name] - usageCount[typeA.name];
              if (0 !== usageCountDiff) {
                return usageCountDiff;
              }
              if (isInterfaceType(typeA) && schema.isSubType(typeA, typeB)) {
                return -1;
              }
              if (isInterfaceType(typeB) && schema.isSubType(typeB, typeA)) {
                return 1;
              }
              return naturalCompare(typeA.name, typeB.name);
            })).map((x => x.name));
          }(schema, type, fieldName));
          if ("" === suggestion) {
            suggestion = didYouMean(function getSuggestedFieldNames(type, fieldName) {
              if (isObjectType(type) || isInterfaceType(type)) {
                const possibleFieldNames = Object.keys(type.getFields());
                return suggestionList(fieldName, possibleFieldNames);
              }
              return [];
            }(type, fieldName));
          }
          context.reportError(new GraphQLError(`Cannot query field "${fieldName}" on type "${type.name}".` + suggestion, {
            nodes: node
          }));
        }
      }
    }
  };
}

export { FieldsOnCorrectTypeRule };
