var jsutils_invariant = require("../../jsutils/invariant.js");

var language_ast = require("../../language/ast.js");

var language_directiveLocation = require("../../language/directiveLocation.js");

var type_directives = require("../../type/directives.js");

var graphql_web = require("@0no-co/graphql.web");

exports.KnownDirectivesRule = function KnownDirectivesRule(context) {
  const locationsMap = Object.create(null);
  const schema = context.getSchema();
  const definedDirectives = schema ? schema.getDirectives() : type_directives.specifiedDirectives;
  for (const directive of definedDirectives) {
    locationsMap[directive.name] = directive.locations;
  }
  const astDefinitions = context.getDocument().definitions;
  for (const def of astDefinitions) {
    if (def.kind === graphql_web.Kind.DIRECTIVE_DEFINITION) {
      locationsMap[def.name.value] = def.locations.map((name => name.value));
    }
  }
  return {
    Directive(node, _key, _parent, _path, ancestors) {
      const name = node.name.value;
      const locations = locationsMap[name];
      if (!locations) {
        context.reportError(new graphql_web.GraphQLError(`Unknown directive "@${name}".`, {
          nodes: node
        }));
        return;
      }
      const candidateLocation = function getDirectiveLocationForASTPath(ancestors) {
        const appliedTo = ancestors[ancestors.length - 1];
        "kind" in appliedTo || jsutils_invariant.invariant(!1);
        switch (appliedTo.kind) {
         case graphql_web.Kind.OPERATION_DEFINITION:
          return function getDirectiveLocationForOperation(operation) {
            switch (operation) {
             case language_ast.OperationTypeNode.QUERY:
              return language_directiveLocation.DirectiveLocation.QUERY;

             case language_ast.OperationTypeNode.MUTATION:
              return language_directiveLocation.DirectiveLocation.MUTATION;

             case language_ast.OperationTypeNode.SUBSCRIPTION:
              return language_directiveLocation.DirectiveLocation.SUBSCRIPTION;
            }
          }(appliedTo.operation);

         case graphql_web.Kind.FIELD:
          return language_directiveLocation.DirectiveLocation.FIELD;

         case graphql_web.Kind.FRAGMENT_SPREAD:
          return language_directiveLocation.DirectiveLocation.FRAGMENT_SPREAD;

         case graphql_web.Kind.INLINE_FRAGMENT:
          return language_directiveLocation.DirectiveLocation.INLINE_FRAGMENT;

         case graphql_web.Kind.FRAGMENT_DEFINITION:
          return language_directiveLocation.DirectiveLocation.FRAGMENT_DEFINITION;

         case graphql_web.Kind.VARIABLE_DEFINITION:
          return language_directiveLocation.DirectiveLocation.VARIABLE_DEFINITION;

         case graphql_web.Kind.SCHEMA_DEFINITION:
         case graphql_web.Kind.SCHEMA_EXTENSION:
          return language_directiveLocation.DirectiveLocation.SCHEMA;

         case graphql_web.Kind.SCALAR_TYPE_DEFINITION:
         case graphql_web.Kind.SCALAR_TYPE_EXTENSION:
          return language_directiveLocation.DirectiveLocation.SCALAR;

         case graphql_web.Kind.OBJECT_TYPE_DEFINITION:
         case graphql_web.Kind.OBJECT_TYPE_EXTENSION:
          return language_directiveLocation.DirectiveLocation.OBJECT;

         case graphql_web.Kind.FIELD_DEFINITION:
          return language_directiveLocation.DirectiveLocation.FIELD_DEFINITION;

         case graphql_web.Kind.INTERFACE_TYPE_DEFINITION:
         case graphql_web.Kind.INTERFACE_TYPE_EXTENSION:
          return language_directiveLocation.DirectiveLocation.INTERFACE;

         case graphql_web.Kind.UNION_TYPE_DEFINITION:
         case graphql_web.Kind.UNION_TYPE_EXTENSION:
          return language_directiveLocation.DirectiveLocation.UNION;

         case graphql_web.Kind.ENUM_TYPE_DEFINITION:
         case graphql_web.Kind.ENUM_TYPE_EXTENSION:
          return language_directiveLocation.DirectiveLocation.ENUM;

         case graphql_web.Kind.ENUM_VALUE_DEFINITION:
          return language_directiveLocation.DirectiveLocation.ENUM_VALUE;

         case graphql_web.Kind.INPUT_OBJECT_TYPE_DEFINITION:
         case graphql_web.Kind.INPUT_OBJECT_TYPE_EXTENSION:
          return language_directiveLocation.DirectiveLocation.INPUT_OBJECT;

         case graphql_web.Kind.INPUT_VALUE_DEFINITION:
          {
            const parentNode = ancestors[ancestors.length - 3];
            "kind" in parentNode || jsutils_invariant.invariant(!1);
            return parentNode.kind === graphql_web.Kind.INPUT_OBJECT_TYPE_DEFINITION ? language_directiveLocation.DirectiveLocation.INPUT_FIELD_DEFINITION : language_directiveLocation.DirectiveLocation.ARGUMENT_DEFINITION;
          }

         default:
          jsutils_invariant.invariant(!1);
        }
      }(ancestors);
      if (candidateLocation && !locations.includes(candidateLocation)) {
        context.reportError(new graphql_web.GraphQLError(`Directive "@${name}" may not be used on ${candidateLocation}.`, {
          nodes: node
        }));
      }
    }
  };
};
