var jsutils_didYouMean = require("../../jsutils/didYouMean.js");

var jsutils_suggestionList = require("../../jsutils/suggestionList.js");

var language_predicates = require("../../language/predicates.js");

var type_introspection = require("../../type/introspection.js");

var type_scalars = require("../../type/scalars.js");

var graphql_web = require("@0no-co/graphql.web");

const standardTypeNames = [ ...type_scalars.specifiedScalarTypes, ...type_introspection.introspectionTypes ].map((type => type.name));

exports.KnownTypeNamesRule = function KnownTypeNamesRule(context) {
  const schema = context.getSchema();
  const existingTypesMap = schema ? schema.getTypeMap() : Object.create(null);
  const definedTypes = Object.create(null);
  for (const def of context.getDocument().definitions) {
    if (language_predicates.isTypeDefinitionNode(def)) {
      definedTypes[def.name.value] = !0;
    }
  }
  const typeNames = [ ...Object.keys(existingTypesMap), ...Object.keys(definedTypes) ];
  return {
    NamedType(node, _1, parent, _2, ancestors) {
      const typeName = node.name.value;
      if (!existingTypesMap[typeName] && !definedTypes[typeName]) {
        var _ancestors$;
        const definitionNode = null !== (_ancestors$ = ancestors[2]) && void 0 !== _ancestors$ ? _ancestors$ : parent;
        const isSDL = null != definitionNode && function isSDLNode(value) {
          return "kind" in value && (language_predicates.isTypeSystemDefinitionNode(value) || language_predicates.isTypeSystemExtensionNode(value));
        }(definitionNode);
        if (isSDL && standardTypeNames.includes(typeName)) {
          return;
        }
        const suggestedTypes = jsutils_suggestionList.suggestionList(typeName, isSDL ? standardTypeNames.concat(typeNames) : typeNames);
        context.reportError(new graphql_web.GraphQLError(`Unknown type "${typeName}".` + jsutils_didYouMean.didYouMean(suggestedTypes), {
          nodes: node
        }));
      }
    }
  };
};
