var graphql_web = require("@0no-co/graphql.web");

exports.NoFragmentCyclesRule = function NoFragmentCyclesRule(context) {
  const visitedFrags = Object.create(null);
  const spreadPath = [];
  const spreadPathIndexByName = Object.create(null);
  return {
    OperationDefinition: () => !1,
    FragmentDefinition(node) {
      detectCycleRecursive(node);
      return !1;
    }
  };
  function detectCycleRecursive(fragment) {
    if (visitedFrags[fragment.name.value]) {
      return;
    }
    const fragmentName = fragment.name.value;
    visitedFrags[fragmentName] = !0;
    const spreadNodes = context.getFragmentSpreads(fragment.selectionSet);
    if (0 === spreadNodes.length) {
      return;
    }
    spreadPathIndexByName[fragmentName] = spreadPath.length;
    for (const spreadNode of spreadNodes) {
      const spreadName = spreadNode.name.value;
      const cycleIndex = spreadPathIndexByName[spreadName];
      spreadPath.push(spreadNode);
      if (void 0 === cycleIndex) {
        const spreadFragment = context.getFragment(spreadName);
        if (spreadFragment) {
          detectCycleRecursive(spreadFragment);
        }
      } else {
        const cyclePath = spreadPath.slice(cycleIndex);
        const viaPath = cyclePath.slice(0, -1).map((s => '"' + s.name.value + '"')).join(", ");
        context.reportError(new graphql_web.GraphQLError(`Cannot spread fragment "${spreadName}" within itself` + ("" !== viaPath ? ` via ${viaPath}.` : "."), {
          nodes: cyclePath
        }));
      }
      spreadPath.pop();
    }
    spreadPathIndexByName[fragmentName] = void 0;
  }
};
