var graphql_web = require("@0no-co/graphql.web");

exports.NoUnusedVariablesRule = function NoUnusedVariablesRule(context) {
  let variableDefs = [];
  return {
    OperationDefinition: {
      enter() {
        variableDefs = [];
      },
      leave(operation) {
        const variableNameUsed = Object.create(null);
        const usages = context.getRecursiveVariableUsages(operation);
        for (const {node} of usages) {
          variableNameUsed[node.name.value] = !0;
        }
        for (const variableDef of variableDefs) {
          const variableName = variableDef.variable.name.value;
          if (!0 !== variableNameUsed[variableName]) {
            context.reportError(new graphql_web.GraphQLError(operation.name ? `Variable "$${variableName}" is never used in operation "${operation.name.value}".` : `Variable "$${variableName}" is never used.`, {
              nodes: variableDef
            }));
          }
        }
      }
    },
    VariableDefinition(def) {
      variableDefs.push(def);
    }
  };
};
