import { inspect } from "../../jsutils/inspect.mjs";

import { isCompositeType } from "../../type/definition.mjs";

import { doTypesOverlap } from "../../utilities/typeComparators.mjs";

import { typeFromAST } from "../../utilities/typeFromAST.mjs";

import { GraphQLError } from "@0no-co/graphql.web";

function PossibleFragmentSpreadsRule(context) {
  return {
    InlineFragment(node) {
      const fragType = context.getType();
      const parentType = context.getParentType();
      if (isCompositeType(fragType) && isCompositeType(parentType) && !doTypesOverlap(context.getSchema(), fragType, parentType)) {
        const parentTypeStr = inspect(parentType);
        const fragTypeStr = inspect(fragType);
        context.reportError(new GraphQLError(`Fragment cannot be spread here as objects of type "${parentTypeStr}" can never be of type "${fragTypeStr}".`, {
          nodes: node
        }));
      }
    },
    FragmentSpread(node) {
      const fragName = node.name.value;
      const fragType = function getFragmentType(context, name) {
        const frag = context.getFragment(name);
        if (frag) {
          const type = typeFromAST(context.getSchema(), frag.typeCondition);
          if (isCompositeType(type)) {
            return type;
          }
        }
      }(context, fragName);
      const parentType = context.getParentType();
      if (fragType && parentType && !doTypesOverlap(context.getSchema(), fragType, parentType)) {
        const parentTypeStr = inspect(parentType);
        const fragTypeStr = inspect(fragType);
        context.reportError(new GraphQLError(`Fragment "${fragName}" cannot be spread here as objects of type "${parentTypeStr}" can never be of type "${fragTypeStr}".`, {
          nodes: node
        }));
      }
    }
  };
}

export { PossibleFragmentSpreadsRule };
