var jsutils_didYouMean = require("../../jsutils/didYouMean.js");

var jsutils_invariant = require("../../jsutils/invariant.js");

var jsutils_suggestionList = require("../../jsutils/suggestionList.js");

var language_predicates = require("../../language/predicates.js");

var type_definition = require("../../type/definition.js");

var graphql_web = require("@0no-co/graphql.web");

const defKindToExtKind = {
  [graphql_web.Kind.SCALAR_TYPE_DEFINITION]: graphql_web.Kind.SCALAR_TYPE_EXTENSION,
  [graphql_web.Kind.OBJECT_TYPE_DEFINITION]: graphql_web.Kind.OBJECT_TYPE_EXTENSION,
  [graphql_web.Kind.INTERFACE_TYPE_DEFINITION]: graphql_web.Kind.INTERFACE_TYPE_EXTENSION,
  [graphql_web.Kind.UNION_TYPE_DEFINITION]: graphql_web.Kind.UNION_TYPE_EXTENSION,
  [graphql_web.Kind.ENUM_TYPE_DEFINITION]: graphql_web.Kind.ENUM_TYPE_EXTENSION,
  [graphql_web.Kind.INPUT_OBJECT_TYPE_DEFINITION]: graphql_web.Kind.INPUT_OBJECT_TYPE_EXTENSION
};

exports.PossibleTypeExtensionsRule = function PossibleTypeExtensionsRule(context) {
  const schema = context.getSchema();
  const definedTypes = Object.create(null);
  for (const def of context.getDocument().definitions) {
    if (language_predicates.isTypeDefinitionNode(def)) {
      definedTypes[def.name.value] = def;
    }
  }
  return {
    ScalarTypeExtension: checkExtension,
    ObjectTypeExtension: checkExtension,
    InterfaceTypeExtension: checkExtension,
    UnionTypeExtension: checkExtension,
    EnumTypeExtension: checkExtension,
    InputObjectTypeExtension: checkExtension
  };
  function checkExtension(node) {
    const typeName = node.name.value;
    const defNode = definedTypes[typeName];
    const existingType = null == schema ? void 0 : schema.getType(typeName);
    let expectedKind;
    if (defNode) {
      expectedKind = defKindToExtKind[defNode.kind];
    } else if (existingType) {
      expectedKind = function typeToExtKind(type) {
        if (type_definition.isScalarType(type)) {
          return graphql_web.Kind.SCALAR_TYPE_EXTENSION;
        }
        if (type_definition.isObjectType(type)) {
          return graphql_web.Kind.OBJECT_TYPE_EXTENSION;
        }
        if (type_definition.isInterfaceType(type)) {
          return graphql_web.Kind.INTERFACE_TYPE_EXTENSION;
        }
        if (type_definition.isUnionType(type)) {
          return graphql_web.Kind.UNION_TYPE_EXTENSION;
        }
        if (type_definition.isEnumType(type)) {
          return graphql_web.Kind.ENUM_TYPE_EXTENSION;
        }
        if (type_definition.isInputObjectType(type)) {
          return graphql_web.Kind.INPUT_OBJECT_TYPE_EXTENSION;
        }
        jsutils_invariant.invariant(!1);
      }(existingType);
    }
    if (expectedKind) {
      if (expectedKind !== node.kind) {
        const kindStr = function extensionKindToTypeName(kind) {
          switch (kind) {
           case graphql_web.Kind.SCALAR_TYPE_EXTENSION:
            return "scalar";

           case graphql_web.Kind.OBJECT_TYPE_EXTENSION:
            return "object";

           case graphql_web.Kind.INTERFACE_TYPE_EXTENSION:
            return "interface";

           case graphql_web.Kind.UNION_TYPE_EXTENSION:
            return "union";

           case graphql_web.Kind.ENUM_TYPE_EXTENSION:
            return "enum";

           case graphql_web.Kind.INPUT_OBJECT_TYPE_EXTENSION:
            return "input object";

           default:
            jsutils_invariant.invariant(!1);
          }
        }(node.kind);
        context.reportError(new graphql_web.GraphQLError(`Cannot extend non-${kindStr} type "${typeName}".`, {
          nodes: defNode ? [ defNode, node ] : node
        }));
      }
    } else {
      const allTypeNames = Object.keys({
        ...definedTypes,
        ...null == schema ? void 0 : schema.getTypeMap()
      });
      const suggestedTypes = jsutils_suggestionList.suggestionList(typeName, allTypeNames);
      context.reportError(new graphql_web.GraphQLError(`Cannot extend type "${typeName}" because it is not defined.` + jsutils_didYouMean.didYouMean(suggestedTypes), {
        nodes: node.name
      }));
    }
  }
};
