import { inspect } from "../../jsutils/inspect.mjs";

import { keyMap } from "../../jsutils/keyMap.mjs";

import { isRequiredArgument, isType } from "../../type/definition.mjs";

import { specifiedDirectives } from "../../type/directives.mjs";

import { GraphQLError, Kind, print } from "@0no-co/graphql.web";

function ProvidedRequiredArgumentsRule(context) {
  return {
    ...ProvidedRequiredArgumentsOnDirectivesRule(context),
    Field: {
      leave(fieldNode) {
        var _fieldNode$arguments;
        const fieldDef = context.getFieldDef();
        if (!fieldDef) {
          return !1;
        }
        const providedArgs = new Set(null === (_fieldNode$arguments = fieldNode.arguments) || void 0 === _fieldNode$arguments ? void 0 : _fieldNode$arguments.map((arg => arg.name.value)));
        for (const argDef of fieldDef.args) {
          if (!providedArgs.has(argDef.name) && isRequiredArgument(argDef)) {
            const argTypeStr = inspect(argDef.type);
            context.reportError(new GraphQLError(`Field "${fieldDef.name}" argument "${argDef.name}" of type "${argTypeStr}" is required, but it was not provided.`, {
              nodes: fieldNode
            }));
          }
        }
      }
    }
  };
}

function ProvidedRequiredArgumentsOnDirectivesRule(context) {
  var _schema$getDirectives;
  const requiredArgsMap = Object.create(null);
  const schema = context.getSchema();
  const definedDirectives = null !== (_schema$getDirectives = null == schema ? void 0 : schema.getDirectives()) && void 0 !== _schema$getDirectives ? _schema$getDirectives : specifiedDirectives;
  for (const directive of definedDirectives) {
    requiredArgsMap[directive.name] = keyMap(directive.args.filter(isRequiredArgument), (arg => arg.name));
  }
  const astDefinitions = context.getDocument().definitions;
  for (const def of astDefinitions) {
    if (def.kind === Kind.DIRECTIVE_DEFINITION) {
      var _def$arguments;
      const argNodes = null !== (_def$arguments = def.arguments) && void 0 !== _def$arguments ? _def$arguments : [];
      requiredArgsMap[def.name.value] = keyMap(argNodes.filter(isRequiredArgumentNode), (arg => arg.name.value));
    }
  }
  return {
    Directive: {
      leave(directiveNode) {
        const directiveName = directiveNode.name.value;
        const requiredArgs = requiredArgsMap[directiveName];
        if (requiredArgs) {
          var _directiveNode$argume;
          const argNodes = null !== (_directiveNode$argume = directiveNode.arguments) && void 0 !== _directiveNode$argume ? _directiveNode$argume : [];
          const argNodeMap = new Set(argNodes.map((arg => arg.name.value)));
          for (const [argName, argDef] of Object.entries(requiredArgs)) {
            if (!argNodeMap.has(argName)) {
              const argType = isType(argDef.type) ? inspect(argDef.type) : print(argDef.type);
              context.reportError(new GraphQLError(`Directive "@${directiveName}" argument "${argName}" of type "${argType}" is required, but it was not provided.`, {
                nodes: directiveNode
              }));
            }
          }
        }
      }
    }
  };
}

function isRequiredArgumentNode(arg) {
  return arg.type.kind === Kind.NON_NULL_TYPE && null == arg.defaultValue;
}

export { ProvidedRequiredArgumentsOnDirectivesRule, ProvidedRequiredArgumentsRule };
