var execution_collectFields = require("../../execution/collectFields.js");

var graphql_web = require("@0no-co/graphql.web");

exports.SingleFieldSubscriptionsRule = function SingleFieldSubscriptionsRule(context) {
  return {
    OperationDefinition(node) {
      if ("subscription" === node.operation) {
        const schema = context.getSchema();
        const subscriptionType = schema.getSubscriptionType();
        if (subscriptionType) {
          const operationName = node.name ? node.name.value : null;
          const variableValues = Object.create(null);
          const document = context.getDocument();
          const fragments = Object.create(null);
          for (const definition of document.definitions) {
            if (definition.kind === graphql_web.Kind.FRAGMENT_DEFINITION) {
              fragments[definition.name.value] = definition;
            }
          }
          const fields = execution_collectFields.collectFields(schema, fragments, variableValues, subscriptionType, node.selectionSet);
          if (fields.size > 1) {
            const extraFieldSelections = [ ...fields.values() ].slice(1).flat();
            context.reportError(new graphql_web.GraphQLError(null != operationName ? `Subscription "${operationName}" must select only one top level field.` : "Anonymous Subscription must select only one top level field.", {
              nodes: extraFieldSelections
            }));
          }
          for (const fieldNodes of fields.values()) {
            if (fieldNodes[0].name.value.startsWith("__")) {
              context.reportError(new graphql_web.GraphQLError(null != operationName ? `Subscription "${operationName}" must not select an introspection top level field.` : "Anonymous Subscription must not select an introspection top level field.", {
                nodes: fieldNodes
              }));
            }
          }
        }
      }
    }
  };
};
