var language_predicates = require("../../language/predicates.js");

var type_directives = require("../../type/directives.js");

var graphql_web = require("@0no-co/graphql.web");

exports.UniqueDirectivesPerLocationRule = function UniqueDirectivesPerLocationRule(context) {
  const uniqueDirectiveMap = Object.create(null);
  const schema = context.getSchema();
  const definedDirectives = schema ? schema.getDirectives() : type_directives.specifiedDirectives;
  for (const directive of definedDirectives) {
    uniqueDirectiveMap[directive.name] = !directive.isRepeatable;
  }
  const astDefinitions = context.getDocument().definitions;
  for (const def of astDefinitions) {
    if (def.kind === graphql_web.Kind.DIRECTIVE_DEFINITION) {
      uniqueDirectiveMap[def.name.value] = !def.repeatable;
    }
  }
  const schemaDirectives = Object.create(null);
  const typeDirectivesMap = Object.create(null);
  return {
    enter(node) {
      if (!("directives" in node) || !node.directives) {
        return;
      }
      let seenDirectives;
      if (node.kind === graphql_web.Kind.SCHEMA_DEFINITION || node.kind === graphql_web.Kind.SCHEMA_EXTENSION) {
        seenDirectives = schemaDirectives;
      } else if (language_predicates.isTypeDefinitionNode(node) || language_predicates.isTypeExtensionNode(node)) {
        const typeName = node.name.value;
        seenDirectives = typeDirectivesMap[typeName];
        if (void 0 === seenDirectives) {
          typeDirectivesMap[typeName] = seenDirectives = Object.create(null);
        }
      } else {
        seenDirectives = Object.create(null);
      }
      for (const directive of node.directives) {
        const directiveName = directive.name.value;
        if (uniqueDirectiveMap[directiveName]) {
          if (seenDirectives[directiveName]) {
            context.reportError(new graphql_web.GraphQLError(`The directive "@${directiveName}" can only be used once at this location.`, {
              nodes: [ seenDirectives[directiveName], directive ]
            }));
          } else {
            seenDirectives[directiveName] = directive;
          }
        }
      }
    }
  };
};
