import { invariant } from "../../jsutils/invariant.mjs";

import { GraphQLError } from "@0no-co/graphql.web";

function UniqueInputFieldNamesRule(context) {
  const knownNameStack = [];
  let knownNames = Object.create(null);
  return {
    ObjectValue: {
      enter() {
        knownNameStack.push(knownNames);
        knownNames = Object.create(null);
      },
      leave() {
        const prevKnownNames = knownNameStack.pop();
        prevKnownNames || invariant(!1);
        knownNames = prevKnownNames;
      }
    },
    ObjectField(node) {
      const fieldName = node.name.value;
      if (knownNames[fieldName]) {
        context.reportError(new GraphQLError(`There can be only one input field named "${fieldName}".`, {
          nodes: [ knownNames[fieldName], node.name ]
        }));
      } else {
        knownNames[fieldName] = node.name;
      }
    }
  };
}

export { UniqueInputFieldNamesRule };
