import { GraphQLError } from "@0no-co/graphql.web";

function UniqueOperationNamesRule(context) {
  const knownOperationNames = Object.create(null);
  return {
    OperationDefinition(node) {
      const operationName = node.name;
      if (operationName) {
        if (knownOperationNames[operationName.value]) {
          context.reportError(new GraphQLError(`There can be only one operation named "${operationName.value}".`, {
            nodes: [ knownOperationNames[operationName.value], operationName ]
          }));
        } else {
          knownOperationNames[operationName.value] = operationName;
        }
      }
      return !1;
    },
    FragmentDefinition: () => !1
  };
}

export { UniqueOperationNamesRule };
