var graphql_web = require("@0no-co/graphql.web");

exports.UniqueOperationTypesRule = function UniqueOperationTypesRule(context) {
  const schema = context.getSchema();
  const definedOperationTypes = Object.create(null);
  const existingOperationTypes = schema ? {
    query: schema.getQueryType(),
    mutation: schema.getMutationType(),
    subscription: schema.getSubscriptionType()
  } : {};
  return {
    SchemaDefinition: checkOperationTypes,
    SchemaExtension: checkOperationTypes
  };
  function checkOperationTypes(node) {
    var _node$operationTypes;
    const operationTypesNodes = null !== (_node$operationTypes = node.operationTypes) && void 0 !== _node$operationTypes ? _node$operationTypes : [];
    for (const operationType of operationTypesNodes) {
      const operation = operationType.operation;
      const alreadyDefinedOperationType = definedOperationTypes[operation];
      if (existingOperationTypes[operation]) {
        context.reportError(new graphql_web.GraphQLError(`Type for ${operation} already defined in the schema. It cannot be redefined.`, {
          nodes: operationType
        }));
      } else if (alreadyDefinedOperationType) {
        context.reportError(new graphql_web.GraphQLError(`There can be only one ${operation} type in schema.`, {
          nodes: [ alreadyDefinedOperationType, operationType ]
        }));
      } else {
        definedOperationTypes[operation] = operationType;
      }
    }
    return !1;
  }
};
