import { groupBy } from "../../jsutils/groupBy.mjs";

import { GraphQLError } from "@0no-co/graphql.web";

function UniqueVariableNamesRule(context) {
  return {
    OperationDefinition(operationNode) {
      var _operationNode$variab;
      const variableDefinitions = null !== (_operationNode$variab = operationNode.variableDefinitions) && void 0 !== _operationNode$variab ? _operationNode$variab : [];
      const seenVariableDefinitions = groupBy(variableDefinitions, (node => node.variable.name.value));
      for (const [variableName, variableNodes] of seenVariableDefinitions) {
        if (variableNodes.length > 1) {
          context.reportError(new GraphQLError(`There can be only one variable named "$${variableName}".`, {
            nodes: variableNodes.map((node => node.variable.name))
          }));
        }
      }
    }
  };
}

export { UniqueVariableNamesRule };
