import { isInputType } from "../../type/definition.mjs";

import { typeFromAST } from "../../utilities/typeFromAST.mjs";

import { print, GraphQLError } from "@0no-co/graphql.web";

function VariablesAreInputTypesRule(context) {
  return {
    VariableDefinition(node) {
      const type = typeFromAST(context.getSchema(), node.type);
      if (void 0 !== type && !isInputType(type)) {
        const variableName = node.variable.name.value;
        const typeName = print(node.type);
        context.reportError(new GraphQLError(`Variable "$${variableName}" cannot be non-input type "${typeName}".`, {
          nodes: node.type
        }));
      }
    }
  };
}

export { VariablesAreInputTypesRule };
