var jsutils_inspect = require("../../jsutils/inspect.js");

var type_definition = require("../../type/definition.js");

var utilities_typeComparators = require("../../utilities/typeComparators.js");

var utilities_typeFromAST = require("../../utilities/typeFromAST.js");

var graphql_web = require("@0no-co/graphql.web");

function allowedVariableUsage(schema, varType, varDefaultValue, locationType, locationDefaultValue) {
  if (type_definition.isNonNullType(locationType) && !type_definition.isNonNullType(varType)) {
    if (!(null != varDefaultValue && varDefaultValue.kind !== graphql_web.Kind.NULL) && !(void 0 !== locationDefaultValue)) {
      return !1;
    }
    return utilities_typeComparators.isTypeSubTypeOf(schema, varType, locationType.ofType);
  }
  return utilities_typeComparators.isTypeSubTypeOf(schema, varType, locationType);
}

exports.VariablesInAllowedPositionRule = function VariablesInAllowedPositionRule(context) {
  let varDefMap = Object.create(null);
  return {
    OperationDefinition: {
      enter() {
        varDefMap = Object.create(null);
      },
      leave(operation) {
        const usages = context.getRecursiveVariableUsages(operation);
        for (const {node, type, defaultValue} of usages) {
          const varName = node.name.value;
          const varDef = varDefMap[varName];
          if (varDef && type) {
            const schema = context.getSchema();
            const varType = utilities_typeFromAST.typeFromAST(schema, varDef.type);
            if (varType && !allowedVariableUsage(schema, varType, varDef.defaultValue, type, defaultValue)) {
              const varTypeStr = jsutils_inspect.inspect(varType);
              const typeStr = jsutils_inspect.inspect(type);
              context.reportError(new graphql_web.GraphQLError(`Variable "$${varName}" of type "${varTypeStr}" used in position expecting type "${typeStr}".`, {
                nodes: [ varDef, node ]
              }));
            }
          }
        }
      }
    },
    VariableDefinition(node) {
      varDefMap[node.variable.name.value] = node;
    }
  };
};
