var jsutils_invariant = require("../../../jsutils/invariant.js");

var type_definition = require("../../../type/definition.js");

var graphql_web = require("@0no-co/graphql.web");

exports.NoDeprecatedCustomRule = function NoDeprecatedCustomRule(context) {
  return {
    Field(node) {
      const fieldDef = context.getFieldDef();
      const deprecationReason = null == fieldDef ? void 0 : fieldDef.deprecationReason;
      if (fieldDef && null != deprecationReason) {
        const parentType = context.getParentType();
        null != parentType || jsutils_invariant.invariant(!1);
        context.reportError(new graphql_web.GraphQLError(`The field ${parentType.name}.${fieldDef.name} is deprecated. ${deprecationReason}`, {
          nodes: node
        }));
      }
    },
    Argument(node) {
      const argDef = context.getArgument();
      const deprecationReason = null == argDef ? void 0 : argDef.deprecationReason;
      if (argDef && null != deprecationReason) {
        const directiveDef = context.getDirective();
        if (null != directiveDef) {
          context.reportError(new graphql_web.GraphQLError(`Directive "@${directiveDef.name}" argument "${argDef.name}" is deprecated. ${deprecationReason}`, {
            nodes: node
          }));
        } else {
          const parentType = context.getParentType();
          const fieldDef = context.getFieldDef();
          null != parentType && null != fieldDef || jsutils_invariant.invariant(!1);
          context.reportError(new graphql_web.GraphQLError(`Field "${parentType.name}.${fieldDef.name}" argument "${argDef.name}" is deprecated. ${deprecationReason}`, {
            nodes: node
          }));
        }
      }
    },
    ObjectField(node) {
      const inputObjectDef = type_definition.getNamedType(context.getParentInputType());
      if (type_definition.isInputObjectType(inputObjectDef)) {
        const inputFieldDef = inputObjectDef.getFields()[node.name.value];
        const deprecationReason = null == inputFieldDef ? void 0 : inputFieldDef.deprecationReason;
        if (null != deprecationReason) {
          context.reportError(new graphql_web.GraphQLError(`The input field ${inputObjectDef.name}.${inputFieldDef.name} is deprecated. ${deprecationReason}`, {
            nodes: node
          }));
        }
      }
    },
    EnumValue(node) {
      const enumValueDef = context.getEnumValue();
      const deprecationReason = null == enumValueDef ? void 0 : enumValueDef.deprecationReason;
      if (enumValueDef && null != deprecationReason) {
        const enumTypeDef = type_definition.getNamedType(context.getInputType());
        null != enumTypeDef || jsutils_invariant.invariant(!1);
        context.reportError(new graphql_web.GraphQLError(`The enum value "${enumTypeDef.name}.${enumValueDef.name}" is deprecated. ${deprecationReason}`, {
          nodes: node
        }));
      }
    }
  };
};
